@@ -4,12 +4,11 @@
 ##       )
 
 setAs("dgTMatrix", "dgeMatrix",
-      function(from) .Call(dgTMatrix_to_dgeMatrix, from)
-      )
+      function(from) .Call(dgTMatrix_to_dgeMatrix, from))
 
 setAs("dgTMatrix", "matrix",
-      function(from) .Call(dgTMatrix_to_matrix, from)
-      )
+      function(from) .Call(dgTMatrix_to_matrix, from))
+
 
 setAs("dgeMatrix", "dgTMatrix",
       function(from) as(as(from, "dgCMatrix"), "dgTMatrix"))
@@ -36,11 +35,7 @@ setAs("dgTMatrix", "dsTMatrix",
 	  else stop("not a symmetric matrix")})
 
 setAs("dgTMatrix", "dtTMatrix",
-      function(from) {
-	  if(isTr <- isTriangular(from))
-	      gt2tT(from, uplo = attr(isTr, "kind"),
-		    diag = "N") ## improve: also test for unit diagonal
-	  else stop("not a triangular matrix")})
+      function(from) check.gt2tT(from, getClassDef("dgTMatrix")))
 
 setAs("matrix", "dgTMatrix",
       function(from) {
