@@ -35,10 +35,7 @@ setMethod("crossprod", signature(x = "lsparseMatrix", y = "lsparseMatrix"),
 
 setMethod("!", "lsparseMatrix",
           ## turns FALSE to TRUE --> dense matrix
-          function(e1) !as(e1, "lgeMatrix"))
-
-setMethod("diag", signature(x = "lsparseMatrix"),
-	  function(x, nrow, ncol = n) diag(as(x, "lgCMatrix")))
+          function(e1) !as(e1, "denseMatrix"))# was "lgeMatrix"
 
 
 setMethod("all", signature(x = "lsparseMatrix"),
