@@ -30,8 +30,13 @@ setAs("pMatrix", "ngTMatrix",
               Dim = d, Dimnames = from@Dimnames)
       })
 
-setAs("pMatrix", "TsparseMatrix",
-      function(from) as(from, "ngTMatrix"))
+setAs("pMatrix", "TsparseMatrix", function(from) as(from, "ngTMatrix"))
+setAs("pMatrix", "nMatrix",	  function(from) as(from, "ngTMatrix"))
+setAs("pMatrix", "lMatrix", function(from) as(as(from, "nMatrix"),"lMatrix"))
+
+setAs("pMatrix", "CsparseMatrix",
+      function(from) as(as(from, "ngTMatrix"), "CsparseMatrix"))
+
 
 setMethod("solve", signature(a = "pMatrix", b = "missing"),
 	  function(a, b) {
