@@ -4,21 +4,24 @@
 
 SEXP Tsparse_validate(SEXP x)
 {
+    /* NB: we do *NOT* check a potential 'x' slot here, at all */
     SEXP
 	islot = GET_SLOT(x, Matrix_iSym),
 	jslot = GET_SLOT(x, Matrix_jSym),
 	dimslot = GET_SLOT(x, Matrix_DimSym);
     int j,
-	*dims = INTEGER(dimslot),
-	ncol, nrow, nnz = length(islot),
+	nrow = INTEGER(dimslot)[0],
+	ncol = INTEGER(dimslot)[1],
+	nnz = length(islot),
 	*xj = INTEGER(jslot),
 	*xi = INTEGER(islot);
 
     if (length(jslot) != nnz)
 	return mkString(_("lengths of slots i and j must match"));
+    /* FIXME: this is checked in super class -- no need to do here: */
     if (length(dimslot) != 2)
 	return mkString(_("slot Dim must have length 2"));
-    nrow = dims[0]; ncol = dims[1];
+
     for (j = 0; j < nnz; j++) {
 	if (xi[j] < 0 || xi[j] >= nrow)
 	    return mkString(_("all row indices must be between 0 and nrow-1"));
@@ -33,7 +36,7 @@ SEXP Tsparse_to_Csparse(SEXP x, SEXP tri)
     cholmod_triplet *chxt = as_cholmod_triplet(x);
     cholmod_sparse *chxs = cholmod_triplet_to_sparse(chxt, chxt->nnz, &c);
     int uploT = 0; char *diag = "";
-    int Rkind = (chxt->xtype == CHOLMOD_REAL) ? Real_kind(x) : 0;
+    int Rkind = (chxt->xtype != CHOLMOD_PATTERN) ? Real_kind(x) : 0;
 
     Free(chxt);
     if (asLogical(tri)) {	/* triangular sparse matrices */
@@ -42,6 +45,6 @@ SEXP Tsparse_to_Csparse(SEXP x, SEXP tri)
 	diag = CHAR(asChar(GET_SLOT(x, Matrix_diagSym)));
     }
     return chm_sparse_to_SEXP(chxs, 1, uploT, Rkind, diag,
-			      duplicate(GET_SLOT(x, Matrix_DimNamesSym)));
+			      GET_SLOT(x, Matrix_DimNamesSym));
 }
 
