@@ -8,7 +8,7 @@ cholmod_common c;
  * the result should *not* be freed with cholmod_sparse_free.  Use
  * Free on the result.
  *
- * @param x pointer to an object that inherits from TsparseMatrix
+ * @param x pointer to an object that inherits from CsparseMatrix
  *
  * @return pointer to a cholmod_triplet object that contains pointers
  * to the slots of x.
@@ -21,8 +21,9 @@ cholmod_sparse *as_cholmod_sparse(SEXP x)
 		     "ngCMatrix", "nsCMatrix", "ntCMatrix",
 		     "zgCMatrix", "zsCMatrix", "ztCMatrix",
 		     ""};
-    int *dims, ctype = Matrix_check_class(class_P(x), valid);
-    SEXP islot;
+    int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
+	ctype = Matrix_check_class(class_P(x), valid);
+    SEXP islot = GET_SLOT(x, Matrix_iSym);
 
     if (ctype < 0) error("invalid class of object to as_cholmod_sparse");
 				/* characteristics of the system */
@@ -32,10 +33,9 @@ cholmod_sparse *as_cholmod_sparse(SEXP x)
     ans->sorted = TRUE;
     ans->x = ans->z = ans->nz = (void *) NULL;
 				/* dimensions and nzmax */
-    dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
     ans->nrow = dims[0];
     ans->ncol = dims[1];
-    islot = GET_SLOT(x, Matrix_iSym);
+
     ans->nzmax = LENGTH(islot);
 				/* slots always present */
     ans->i = (void *) INTEGER(islot);
@@ -310,38 +310,42 @@ SEXP chm_triplet_to_SEXP(cholmod_triplet *a, int dofree, int uploT, int Rkind,
  */
 cholmod_dense *as_cholmod_dense(SEXP x)
 {
-    cholmod_dense *ans = Calloc(1, cholmod_dense);
-    char *valid[] = {"dmatrix", "dgeMatrix",
-		     "lmatrix", "lgeMatrix",
-		     "nmatrix", "ngeMatrix",
-		     "zmatrix", "zgeMatrix", ""};
-    int dims[2], ctype = Matrix_check_class(class_P(x), valid), nprot = 0;
+#define _AS_cholmod_dense_1						\
+    cholmod_dense *ans = Calloc(1, cholmod_dense);			\
+    char *valid[] = {"dmatrix", "dgeMatrix",				\
+		     "lmatrix", "lgeMatrix",				\
+		     "nmatrix", "ngeMatrix",				\
+		     "zmatrix", "zgeMatrix", ""};			\
+    int dims[2], ctype = Matrix_check_class(class_P(x), valid), nprot = 0; \
+									\
+    if (ctype < 0) {		/* not a classed matrix */		\
+	if (isMatrix(x)) Memcpy(dims, INTEGER(getAttrib(x, R_DimSymbol)), 2); \
+	else {dims[0] = LENGTH(x); dims[1] = 1;}			\
+	if (isInteger(x)) {						\
+	    x = PROTECT(coerceVector(x, REALSXP));			\
+	    nprot++;							\
+	}								\
+	ctype = (isReal(x) ? 0 :					\
+		 (isLogical(x) ? 2 : /* logical -> default to "l", not "n" */ \
+		  (isComplex(x) ? 6 : -1)));				\
+    } else Memcpy(dims, INTEGER(GET_SLOT(x, Matrix_DimSym)), 2);	\
+    if (ctype < 0) error("invalid class of object to as_cholmod_dense"); \
+				/* characteristics of the system */	\
+    ans->dtype = CHOLMOD_DOUBLE;					\
+    ans->x = ans->z = (void *) NULL;					\
+				/* dimensions and nzmax */		\
+    ans->d = ans->nrow = dims[0];					\
+    ans->ncol = dims[1];						\
+    ans->nzmax = dims[0] * dims[1];					\
+				/* set the xtype and any elements */	\
+    switch(ctype / 2) {							\
+    case 0: /* "d" */							\
+	ans->xtype = CHOLMOD_REAL;					\
+	ans->x = (void *) REAL((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x); \
+	break
+
+    _AS_cholmod_dense_1;
 
-    if (ctype < 0) {		/* not a classed matrix */
-	if (isMatrix(x)) Memcpy(dims, INTEGER(getAttrib(x, R_DimSymbol)), 2);
-	else {dims[0] = LENGTH(x); dims[1] = 1;}
-	if (isInteger(x)) {
-	    x = PROTECT(coerceVector(x, REALSXP));
-	    nprot++;
-	}
-	ctype = (isReal(x) ? 0 :
-		 (isLogical(x) ? 2 : /* logical -> default to "l", not "n" */
-		  (isComplex(x) ? 6 : -1)));
-    } else Memcpy(dims, INTEGER(GET_SLOT(x, Matrix_DimSym)), 2);
-    if (ctype < 0) error("invalid class of object to as_cholmod_dense");
-				/* characteristics of the system */
-    ans->dtype = CHOLMOD_DOUBLE;
-    ans->x = ans->z = (void *) NULL;
-				/* dimensions and nzmax */
-    ans->d = ans->nrow = dims[0];
-    ans->ncol = dims[1];
-    ans->nzmax = dims[0] * dims[1];
-				/* set the xtype and any elements */
-    switch(ctype / 2) {
-    case 0: /* "d" */
-	ans->xtype = CHOLMOD_REAL;
-	ans->x = (void *) REAL((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x);
-	break;
     case 1: /* "l" */
 	ans->xtype = CHOLMOD_REAL;
 	ans->x =
@@ -352,15 +356,40 @@ cholmod_dense *as_cholmod_dense(SEXP x)
 	ans->xtype = CHOLMOD_PATTERN;
 	ans->x = (void *) LOGICAL((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x);
 	break;
-    case 3: /* "z" */
-	ans->xtype = CHOLMOD_COMPLEX;
-	ans->x = (void *) COMPLEX((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x);
-	break;
-    }
-    UNPROTECT(nprot);
-    return ans;
+
+#define _AS_cholmod_dense_2						\
+    case 3: /* "z" */							\
+	ans->xtype = CHOLMOD_COMPLEX;					\
+	ans->x = (void *) COMPLEX((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x); \
+	break;								\
+    }									\
+    UNPROTECT(nprot);							\
+    return ans
+
+    _AS_cholmod_dense_2;
 }
 
+/* version of as_cholmod_dense() that produces a cholmod_dense matrix
+ * with REAL 'x' slot -- i.e. treats "nMatrix" as "lMatrix" -- as only difference;
+ * Not just via a flag in as_cholmod_dense() since that has fixed API */
+cholmod_dense *as_cholmod_x_dense(SEXP x)
+{
+    _AS_cholmod_dense_1;
+
+    case 1: /* "l" */
+    case 2: /* "n" (no NA in 'x', but *has* 'x' slot => treat as "l" */
+	ans->xtype = CHOLMOD_REAL;
+	ans->x =
+	    (void *) REAL(coerceVector((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x,
+				       REALSXP));
+	break;
+
+    _AS_cholmod_dense_2;
+}
+
+#undef _AS_cholmod_dense_1
+#undef _AS_cholmod_dense_2
+
 void R_cholmod_error(int status, char *file, int line, char *message)
 {
     error(_("Cholmod error `%s' at file:%s, line %d"), message, file, line);
@@ -406,10 +435,13 @@ SEXP chm_dense_to_SEXP(cholmod_dense *a, int dofree, int Rkind, SEXP dn)
     PROTECT(dn); /* << (why? -- just cut&paste from chm_dense_to_mat.. below*/
 
     switch(a->xtype) {		/* determine the class of the result */
-    case CHOLMOD_PATTERN:
-	cl = "ngeMatrix"; break;
+/* CHOLMOD_PATTERN never happens because cholmod_dense can't :
+ *     case CHOLMOD_PATTERN:
+ * 	cl = "ngeMatrix"; break;
+ */
     case CHOLMOD_REAL:
-	switch(Rkind) {
+	switch(Rkind) { /* -1: special for this function! */
+	case -1: cl = "ngeMatrix"; break;
 	case 0:	 cl = "dgeMatrix"; break;
 	case 1:	 cl = "lgeMatrix"; break;
 	default: error("unknown 'Rkind'");
@@ -426,7 +458,7 @@ SEXP chm_dense_to_SEXP(cholmod_dense *a, int dofree, int Rkind, SEXP dn)
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = a->nrow; dims[1] = a->ncol;
     ntot = dims[0] * dims[1];
-    if (a->d == a->nrow) {	/* copy data slot if present */
+    if (a->d == a->nrow) {	/* copy data slot -- always present in dense(!) */
 	if (a->xtype == CHOLMOD_REAL) {
 	    int i, *m_x;
 	    switch(Rkind) {
@@ -434,7 +466,8 @@ SEXP chm_dense_to_SEXP(cholmod_dense *a, int dofree, int Rkind, SEXP dn)
 		Memcpy(REAL(ALLOC_SLOT(ans, Matrix_xSym, REALSXP, ntot)),
 		       (double *) a->x, ntot);
 		break;
-	    case 1:
+	    case -1: /* nge*/
+	    case 1:  /* lge*/
 		m_x = LOGICAL(ALLOC_SLOT(ans, Matrix_xSym, LGLSXP, ntot));
 		for (i=0; i < ntot; i++)
 		    m_x[i] = (int) ((double *) a->x)[i];
